@@ -9,15 +9,21 @@ from functools import partial
 from types import FunctionType
 from typing import Any, Callable, Generic, Mapping
 
-from pydantic_core import PydanticUndefined, SchemaSerializer, SchemaValidator
+from pydantic_core import PydanticUndefined, SchemaSerializer
 from typing_extensions import dataclass_transform, deprecated
 
 from ..errors import PydanticUndefinedAnnotation, PydanticUserError
 from ..fields import Field, FieldInfo, ModelPrivateAttr, PrivateAttr
+from ..plugin._schema_validator import create_schema_validator
 from ..warnings import PydanticDeprecatedSince20
 from ._config import ConfigWrapper
-from ._core_utils import collect_invalid_schemas, flatten_schema_defs, inline_schema_defs
-from ._decorators import ComputedFieldInfo, DecoratorInfos, PydanticDescriptorProxy
+from ._core_utils import collect_invalid_schemas, simplify_schema_references, validate_core_schema
+from ._decorators import (
+    ComputedFieldInfo,
+    DecoratorInfos,
+    PydanticDescriptorProxy,
+    get_attribute_from_bases,
+)
 from ._discriminated_union import apply_discriminators
 from ._fields import collect_model_fields, is_valid_field_name, is_valid_privateattr_name
 from ._generate_schema import GenerateSchema
@@ -99,9 +105,9 @@ class ModelMetaclass(ABCMeta):
                 namespace, config_wrapper.ignored_types, class_vars, base_field_names
             )
             if private_attributes:
-                if 'model_post_init' in namespace:
+                original_model_post_init = get_model_post_init(namespace, bases)
+                if original_model_post_init is not None:
                     # if there are private_attributes and a model_post_init function, we handle both
-                    original_model_post_init = namespace['model_post_init']
 
                     def wrapped_model_post_init(self: BaseModel, __context: Any) -> None:
                         """We need to both initialize private attributes and call the user-defined model_post_init
@@ -261,6 +267,18 @@ def init_private_attributes(self: BaseModel, __context: Any) -> None:
     object_setattr(self, '__pydantic_private__', pydantic_private)
 
 
+def get_model_post_init(namespace: dict[str, Any], bases: tuple[type[Any], ...]) -> Callable[..., Any] | None:
+    """Get the `model_post_init` method from the namespace or the class bases, or `None` if not defined."""
+    if 'model_post_init' in namespace:
+        return namespace['model_post_init']
+
+    from ..main import BaseModel
+
+    model_post_init = get_attribute_from_bases(bases, 'model_post_init')
+    if model_post_init is not BaseModel.model_post_init:
+        return model_post_init
+
+
 def inspect_namespace(  # noqa C901
     namespace: dict[str, Any],
     ignored_types: tuple[type[Any], ...],
@@ -374,16 +392,11 @@ def set_default_hash_func(namespace: dict[str, Any], bases: tuple[type[Any], ...
     if '__hash__' in namespace:
         return
 
-    base_hash_func = None
-    for base in bases:
-        base_hash_func = getattr(base, '__hash__', PydanticUndefined)
-        if base_hash_func is not PydanticUndefined:
-            break
-
-    if base_hash_func is None:
-        # This will be the case for `BaseModel` since it defines `__eq__` but not `__hash__`.
-        # In this case, we generate a standard hash function, generally for use with frozen models.
-
+    base_hash_func = get_attribute_from_bases(bases, '__hash__')
+    if base_hash_func in {None, object.__hash__}:
+        # If `__hash__` is None _or_ `object.__hash__`, we generate a hash function.
+        # It will be `None` if not overridden from BaseModel, but may be `object.__hash__` if there is another
+        # parent class earlier in the bases which doesn't override `__hash__` (e.g. `typing.Generic`).
         def hash_func(self: Any) -> int:
             return hash(self.__class__) + hash(tuple(self.__dict__.values()))
 
@@ -475,16 +488,16 @@ def complete_model_class(
     core_config = config_wrapper.core_config(cls)
 
     schema = gen_schema.collect_definitions(schema)
-    schema = apply_discriminators(flatten_schema_defs(schema))
+
+    schema = apply_discriminators(simplify_schema_references(schema))
     if collect_invalid_schemas(schema):
         set_model_mocks(cls, cls_name)
         return False
 
     # debug(schema)
-    cls.__pydantic_core_schema__ = schema
-    simplified_core_schema = inline_schema_defs(schema)
-    cls.__pydantic_validator__ = SchemaValidator(simplified_core_schema, core_config)
-    cls.__pydantic_serializer__ = SchemaSerializer(simplified_core_schema, core_config)
+    cls.__pydantic_core_schema__ = schema = validate_core_schema(schema)
+    cls.__pydantic_validator__ = create_schema_validator(schema, core_config, config_wrapper.plugin_settings)
+    cls.__pydantic_serializer__ = SchemaSerializer(schema, core_config)
     cls.__pydantic_complete__ = True
 
     # set __signature__ attr only for model class, but not for its instances
